/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.concurrent.Immutable;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.As.NotFoundBehaviour;
import it.tidalwave.util.AsException;
import it.tidalwave.util.spi.AsDelegate;
import it.tidalwave.util.spi.AsDelegateProvider;
import it.tidalwave.netbeans.capabilitiesprovider.LookupFactory;

/***********************************************************************************************************************
 *
 * This class provides an implementation for {@link As} based on the NetBeans Platform {@link Lookup}.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Immutable
@ServiceProvider(service=AsDelegateProvider.class)
public class AsLookupAdapter implements AsDelegateProvider
  {
    @Immutable
    static class LookupAsDelegate implements AsDelegate
      {
        @Nonnull
        private final Lookup lookup;

        public LookupAsDelegate (final @Nonnull Object datum)
          {
            lookup = LookupFactory.createLookup(datum);
          }
        
        @Override @Nonnull
        public <T> T as (final @Nonnull Class<T> type,
                         final @Nonnull NotFoundBehaviour<T> notFoundBehaviour) 
          {
            final T t = lookup.lookup(type);
            return (t != null) ? t : notFoundBehaviour.run(new AsException(type));
          }
      }
    
    @Override @Nonnull
    public AsDelegate createAsDelegate (final @Nonnull Object datum) 
      {
        return new LookupAsDelegate(datum);
      }
  }
