/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.LookupListener;

/***********************************************************************************************************************
 *
 * An implementation of {@link Lookup} that wraps a {@link Provider}. It makes it possible to lazy instantiate the
 * object to provide.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 * @stable
 *
 **********************************************************************************************************************/
public class LookupDelegatingToProvider<T> extends Lookup
  {
    @Nonnull
    private final Class<T> clazz;

    @Nonnull
    private final javax.inject.Provider<T> provider;

    /*******************************************************************************************************************
     *
     * Creates an instance.
     *
     * @param  clazz     the class of the provided object
     * @param  provider  the provider
     *
     ******************************************************************************************************************/
    public LookupDelegatingToProvider (final @Nonnull Class<T> clazz,
                                       final @Nonnull javax.inject.Provider<T> provider)
      {
        this.clazz = clazz;
        this.provider = provider;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public <X> X lookup (final @Nonnull Class<X> clazz)
      {
        if (this.clazz.isAssignableFrom(clazz))
          {
            return (X)provider.get();
          }

        return null;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public <X> Result<X> lookup (final @Nonnull Template<X> template)
      {
        final X object = lookup(template.getType());
        final List<X> result = (object == null) ? Collections.<X>emptyList() : Collections.singletonList(object);
        
        return new Result<X>()
          {
            @Override
            public void addLookupListener (final @Nonnull LookupListener listener)
              {
              }

            @Override
            public void removeLookupListener (final @Nonnull LookupListener listener)
              {
              }

            @Override @Nonnull
            public Collection<? extends X> allInstances()
              {
                return result;
              }
          };
      }
  }
